/********************************************************************
					Delphi Corporation
			Dynamics & Propulsion Innovation Center

	created:	2002/09/20
	filename: 	Z:\Projects\CDKit\Downloader\Downloader.cpp
	file path:	Z:\Projects\CDKit\Downloader
	author:		Carlos Buelna
	
	purpose:	Download HEX files to PIC Flash micros

	This code is based on the HI-TECH's downloader code. It has been
	modified to support both 16F87x and 16F87xA PIC Flash micros.

	History
	2002NOV20: Remove some debug code from release version.
			downloads 32 bytes at the time instead of 16.

*********************************************************************/

// Downloader.cpp: implementation of the CDownloader class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Downloader.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDownloader::CDownloader() {
	m_bCommOpened = FALSE;
	m_bAsync = FALSE;
	m_hCommport = INVALID_HANDLE_VALUE;
	m_bStop = TRUE;
	for ( int i=0;i<ROM_SIZE;i++ ) {
		pd[i].iAddress = MAXDWORD;
		pd[i].data[0] = 0x3F;
		pd[i].data[1] = 0xFF;
	}
}

CDownloader::~CDownloader() {
	_commclose();
}


//////////////////////////////////////////////////////////////////////
// Serial Communication implementation
//////////////////////////////////////////////////////////////////////

void CDownloader::_commclose() {
	if ( m_hCommport ) {
		if ( CloseHandle(m_hCommport) ) {
			m_hCommport = INVALID_HANDLE_VALUE;
			m_bCommOpened = FALSE;
		}
	}
}

DWORD CDownloader::_commopen() {
	DCB dcb;

	if ( m_hCommport != INVALID_HANDLE_VALUE ) {
		return -1;
	}
	m_bCommOpened = FALSE;
	m_hCommport = CreateFile( m_sCommport,
							GENERIC_READ|GENERIC_WRITE,
							0, /*not shared*/
							0, /*no security descriptor*/
							OPEN_EXISTING,
							m_bAsync?FILE_FLAG_OVERLAPPED:0,
							NULL );
	if ( INVALID_HANDLE_VALUE == m_hCommport ) {
		return GetLastError();
	}
	if ( !GetCommState(m_hCommport, &dcb) ) {
		DWORD dwE = GetLastError();
		_commclose();
		return dwE;
	}

	dcb.StopBits = 0;
	dcb.Parity = 0;
	dcb.BaudRate = m_iBaudrate;
	dcb.ByteSize = 8;
	dcb.fDsrSensitivity = FALSE;
	// no flow control
	dcb.fOutxCtsFlow = FALSE;
	dcb.fOutxDsrFlow = FALSE;
	dcb.fOutX = FALSE;
	dcb.fInX = FALSE;
	if ( !SetCommState(m_hCommport, &dcb) ) {
		DWORD dwE = GetLastError();
		_commclose();
		return dwE;
	}

	COMMTIMEOUTS ct;
	if ( !GetCommTimeouts(m_hCommport, &ct) ) {
		DWORD dwE = GetLastError();
		_commclose();
		return dwE;
	}
    ct.ReadIntervalTimeout = 20; //0xFFFFFFFF;
    ct.ReadTotalTimeoutMultiplier = 10;
    ct.ReadTotalTimeoutConstant = 1000;
    ct.WriteTotalTimeoutMultiplier = 10; //2*9600/dwBaud;
    ct.WriteTotalTimeoutConstant = 100;
    if ( !SetCommTimeouts(m_hCommport, &ct) ) {
		DWORD dwE = GetLastError();
		_commclose();
		return dwE;
    }

	m_bCommOpened = TRUE;
	return 0;
}

DWORD CDownloader::_commread( LPVOID pB, DWORD n ) {
	DWORD dwBR;
	ReadFile( m_hCommport, pB, n, &dwBR, NULL );
	return dwBR;
}

DWORD CDownloader::_commwrite( LPVOID pB, DWORD n ) {
	DWORD dwBW;
	WriteFile( m_hCommport, pB, n, &dwBW, NULL );
	return dwBW;
}

void CDownloader::_commpurge( void ) {
	PurgeComm( m_hCommport, PURGE_TXABORT|PURGE_RXABORT|PURGE_TXCLEAR|PURGE_RXCLEAR );
}

BOOL CDownloader::_SendByte( BYTE b ) {
	//Sleep( 10 );
	return (BOOL)_commwrite( &b, 1 );
}

BYTE CDownloader::_GetByte() {
	BYTE rb[1];
	_commread( rb, 1 );
	return rb[0];
}

//////////////////////////////////////////////////////////////////////
// Downloader specific implementation
//////////////////////////////////////////////////////////////////////
void CDownloader::OnLooking( DWORD n ) {
}

void CDownloader::OnDownloadProgress( DWORD n ) {
}

void CDownloader::OnDownloadDone() {
}

void CDownloader::OnDownload() {
}

void CDownloader::OnHEXFileLoaded() {
}

void CDownloader::OnError( DWORD dwE ) {
}

void CDownloader::OnDownloadStart() {
}

void CDownloader::OnContact() {
}

BOOL CDownloader::IsDownloading() {
	return !m_bStop;
}

BOOL CDownloader::_Run() {
	m_bStop = FALSE;
	OnDownloadStart();
	if ( _LoadHexFile() ) {
		if ( _MakeContact() ) {
			if ( _Download() ) {
				_SendByte( DONE );
				_commclose();
				m_bStop = TRUE;
				return m_bStop;
			}
		}
	}
	_commclose();
	m_bStop = TRUE;
	return FALSE;
}

BOOL CDownloader::DownloadStart( LPCTSTR pszHexFile, LPCTSTR pszCOM/*=_T("COM1")*/,
						 int iBaudRate/*=CBR_115200*/,
						 int iTries/*=5*/ ) {
	m_sCommport = pszCOM;
	m_iBaudrate = iBaudRate;
	m_sHEXFile = pszHexFile;
	m_iTries = iTries;
	if ( !_commopen() ) {
		return _Run();
	}
	else {
		return FALSE;
	}
}

void CDownloader::DownloadStop() {
	m_bStop = TRUE;
}

PROGDATA* CDownloader::GetCodeMem() {
	return pd;
}

PROGDATA* CDownloader::LoadHexFile( LPCTSTR lpsHexFile ) {
	m_sHEXFile = lpsHexFile;
	return (PROGDATA*)_LoadHexFile();
}

BOOL CDownloader::_LoadHexFile( void ) {
	CStdioFile cf;
	BYTE* ptr;
	int iCount;
	int iAddr;
	int iRecType;

	m_dwTotal = 0;
	if ( cf.Open(m_sHEXFile, CFile::modeRead|CFile::typeText) ) {
		CString sIn;
		while( cf.ReadString(sIn) ) {
			ptr = (BYTE*)sIn.GetBuffer( 256 );
			if ( *ptr++ != ':' ) {
				OnError( DOWNLOADER_ERROR_BADHEXFORMAT );
				return NULL;
			}
			iCount = g2x( ptr );
			ptr += 2;
			iAddr = g4x( ptr );
			iAddr /= 2;
			ptr += 4;
			iRecType = g2x( ptr );
			ptr += 2;
			if ( 1 == iRecType ) {
				cf.Close();
				OnHEXFileLoaded();
				return (BOOL)pd;
			}
			if ( iRecType > 1 ) {
				OnError( DOWNLOADER_ERROR_CANTHANDLEREC );
				cf.Close();
				return NULL;
			}
			for ( int i=0;i<iCount/2;++i ) {
				pd[m_dwTotal].iAddress = iAddr;
				pd[m_dwTotal].data[1] = g2x( ptr );
				ptr += 2;
				pd[m_dwTotal].data[0] = g2x( ptr );
				if ( pd[m_dwTotal].data[0] > 0x3F ) {
					OnError( DOWNLOADER_ERROR_BADHEXFORMAT );
					cf.Close();
					return NULL;
				}
				ptr += 2;
				++iAddr;
				++m_dwTotal;
			}
		}
		cf.Close();
		OnHEXFileLoaded();
		return (BOOL)pd;
	}
	else {
		return NULL;
	}
}

BOOL CDownloader::_Download( void ) {
	BYTE buff[BUFFER_SIZE]={0};
	BYTE chk,ch,*ptr;
	UINT count, buftot, last, address;

#ifdef _DEBUG
	CStdioFile f;
	f.Open( "c:\\debug.txt", CFile::modeCreate|CFile::modeWrite );
	CString s;
#endif

	OnDownload();
	count = 0;
	while ( count<m_dwTotal && !m_bStop ) {
		FillMemory( buff, BUFFER_SIZE, 0xFF );
		ptr = buff;
		address = pd[count].iAddress;
		buftot = 0;
		chk = 0;
		for ( last=address;((buftot<BUFFER_SIZE)&&(count<m_dwTotal)); ) {
			UINT fillcounter = address&0x03;
			if ( fillcounter ) {
				for ( UINT h=0;h<fillcounter;h++ ) {
					*ptr++ = 0x00;
					chk += 0x00;
					*ptr++ = 0x00;
					chk += 0x00;
					buftot += 2;
				}
				address -= fillcounter;
			}
			*ptr++ = pd[count].data[0];
			chk += pd[count].data[0];
			*ptr++ = pd[count].data[1];
			chk += pd[count].data[1];
			buftot += 2;
			++count;
			++last;
			if ( (pd[count].iAddress) != last ) {
				TRACE1("buftot: %d\n", buftot);
				UINT tt = buftot%8;
				UINT buftot_tmp = buftot;
				if ( tt ) {
					buftot = buftot + 8-tt;
					for ( DWORD k=0;k<(buftot-buftot_tmp)/2;k++ ) {
						*ptr++ = 0x00;
						chk += 0x00;
						*ptr++ = 0x00;
						chk += 0x00;
					}
				}
				break;
			}
		}
		// send the buffer to the PIC
		_SendByte( WRITE );
		_SendByte( address>>8 ); // send the start address
		_SendByte( address );
		_SendByte( buftot ); // send how many bytes to write
		_SendByte( chk ); // send check sum
#ifdef _DEBUG
		s.Format( ":%02X%04X00", buftot, address*2);
		f.WriteString( s );
#endif
		for ( UINT i=0;i<buftot;++i ) {
			_SendByte( buff[i] ); // send the data
#ifdef _DEBUG
			s.Format("%02X", buff[i]);
			f.WriteString( s );
#endif
		}
#ifdef _DEBUG
 		s.Format( "%02X\n", chk );
		f.WriteString( s );
#endif
		ch = _GetByte();
		if ( ch != DATA_OK ) { // data sent ok?
			OnError( DOWNLOADER_ERROR_DATA );
			TRACE0("*** download error: data error ***\n");
			return FALSE;
		}
		ch = _GetByte();
		if ( WRITE_OK != ch ) {
			OnError( DOWNLOADER_ERROR_BADWRITE );
			TRACE0("*** download error: bad write ***\n");
			return FALSE;
		}
		OnDownloadProgress( (DWORD)(((float)count/(float)m_dwTotal)*100.0f) );
		_PeekAndPump();
	}

#ifdef _DEBUG
	f.Close();
#endif
	OnDownloadDone();
	return TRUE;
}

BOOL CDownloader::_MakeContact() {
	char buf[3] = {IDENTIFY,0x01,0x0D};
	char rbuf[2];

	if ( !_commwrite(&buf[2],1) ) {
		return FALSE;
	}
	Sleep( 100 );
	_commpurge();
	// send reset command
	if ( !_commwrite(&buf[1],2) ) {
		return FALSE;
	}
	Sleep( 100 );
	for ( int i=0;i<m_iTries && !m_bStop;i++ ) {
		OnLooking( i );
		_PeekAndPump();
		if ( !_commwrite(buf,1) ) {
			return FALSE;
		}
		_PeekAndPump();
		if ( _commread(rbuf,1) ) {
			if ( rbuf[0] == ID_ACK ) {
				OnContact();
				return TRUE;
			}
		}
		Sleep( 50 );
	}
	return FALSE;
}

void CDownloader::_PeekAndPump() {
	MSG msg;
	if ( ::PeekMessage(&msg, AfxGetMainWnd()->m_hWnd, 0, 0, PM_REMOVE) ) {
		::TranslateMessage( &msg );
		::DispatchMessage( &msg );
	}
}

unsigned char CDownloader::gx( unsigned char c ) {
	if ( c >= 'a' && c <= 'f' ) {
		return c-'a'+10;
	}
	if ( c >= 'A' && c <= 'F' ) {
		return c-'A'+10;
	}
	return c-'0';
}

UINT CDownloader::g2x( unsigned char *s ) {
	return (gx(*s)<<4)+gx(s[1]);
}

UINT CDownloader::g4x( unsigned char *s ) {
	return (g2x(s)<<8)+g2x(s+2);
}

CString CDownloader::UpdateHEXInfo( LPCTSTR fn ) {
	CFileFind ff;
	CString s;
	if ( ff.FindFile(fn) ) {
		ff.FindNextFile();
		CTime ct;
		ff.GetLastWriteTime( ct );
		s = ct.Format( "Last modified: %A, %B %d, %Y at %H:%M:%S." );
		return s;
	}
	return _T("");
}