/*	16F87x BOOT LOADER Copyright (C)1999 HI-TECH Software.
 *	Freely distubutable.
 *	Linker options should be changed to position the bootloader at
 *	at the desired address. ie: 0xxF00 - 0xxFFF
 *	Designed to be < 256 words. Compile with full
 *	optimizations (-Zg -O) to achieve this.

Modified by Shane Tolmie of KeyGhost corporation.  Freely distributable.
Questions and comments to picfaq@keyghost.com
Want to see 4Mb of Hi-Tech C FAQ and sample source code? http://www.keyghost.com/
(also has tiny device to record keystrokes in hardware on PC)

Modified by:
/********************************************************************
					Delphi Corporation
			Dynamics & Propulsion Innovation Center

	created:	2002/09/24 24:9:2002 16:17
	filename: 	picboot.c
	file path:	Z:\projects\CDKit\picboot
	author:		Carlos Buelna (carlos.buelna@delphiauto.com)
	
	purpose:	Bootloader code for Microchip's microcontrollers
				16F87x series.
	comments:	Use HT-TECH PICC compiler v7.86 or greater.
*********************************************************************/

#include "picboot.h"

#if FOSC == 4000000UL
	#define OSC	XT
#else
	#define OSC	HS
#endif
__CONFIG(OSC & WDTEN & PWRTEN & BOREN & LVPDIS & DUNPROT & WRTEN & DEBUGDIS & UNPROTECT);

/********************************************************************
	name:		getchar
	input:		none
	output:		received byte
	used in:	main()
	created:	2002/09/24 24:9:2002 17:17
	author:		Carlos Buelna
	
	purpose:	to read a byte out the serial port using a delay
*********************************************************************/
UBYTE getchar( void ) {
	UINT16 i,j;

	for ( i=0;i<46;i++ ) {
		for ( j=0;j<DELAY_LOOP;j++ ) {
			if ( RCIF ) {
				return RCREG;
			}
		}
	}
	return (UBYTE)0;
}

void putchar( UBYTE byte_to_send ) {
	while ( !TXIF );
	TXREG = byte_to_send;
}

void main ( void ) {
	UBYTE			ui8_EEDATAHTmp;
	UBYTE			ui8_EEDataTmp;
	UBYTE			buff[BUFFER_SIZE];
	UBYTE			ui8_Count;
	UBYTE			ui8_Amount;
	UBYTE			ui8_Checksum1;
	UBYTE			ui8_Checksum2;
	UINT16			ui16_Address;

	//initialize USART for serial communication
	SPBRG = _SPBRG;
	TXSTA = 0x24;
	RCSTA = 0x90;

	for ( ;; ) {
   		CLRWDT();
		switch ( getchar() ) {
		case IDENTIFY:
			putchar( (UBYTE)ID_ACK );
			break;
		case WRITE:
			// get write address (2 bytes: high byte first)
			ui16_Address = (UINT16)(getchar()<<8);
			ui16_Address |= getchar();
			ui8_Amount = getchar();
			// get checksum
			ui8_Checksum2 = getchar();
			ui8_Count = 0;
			ui8_Checksum1 = 0;
			while ( ui8_Count < ui8_Amount ) {
				buff[ui8_Count] = getchar();
				ui8_Checksum1 += buff[ui8_Count++];
			}
			if ( ui8_Checksum1 != ui8_Checksum2 ) {
				putchar( (UBYTE)DATA_FAILED );
				break;
			}
			putchar( (UBYTE)DATA_OK );
			ui8_Count = 0;
			while ( ui8_Count < ui8_Amount ) {
				//move user's start code to a different location
				//if this is EEPROM, high byte is ignored anyway
				if ( ui16_Address < 4 ) {
					EEADRH = USER_START>>8;
				}
				else {
					EEADRH = ui16_Address>>8;
				}
				EEADR = (UBYTE)ui16_Address;
				EEDATH = buff[ui8_Count];
				ui8_EEDATAHTmp = EEDATH;
				EEDATA = buff[ui8_Count+1];
				ui8_EEDataTmp = EEDATA;

				//EEIF = 0;
				EEPGD = 1;
				WREN = 1;
				EECON2 = 0x55;
				EECON2 = 0xAA;
				WR = 1;
				NOP();
				NOP();
				WREN = 0;

				// read data to verify
				RD = 1;
				NOP();
				NOP();
				if ( (EEDATH==ui8_EEDATAHTmp) || (EEDATA==ui8_EEDataTmp) ) {
					ui8_Count += 2;
					ui16_Address++;
				}
			}
			putchar( (UBYTE)WRITE_OK );
			break;
		case DONE:
			putchar( (UBYTE)DONE );
		default:
			TXSTA = 2;
			RCSTA = 0;
			CLRWDT();
	#asm
			ljmp USER_START
	#endasm
		}
	}
}
