#ifndef _PICBOOT_H
#define _PICBOOT_H

#if !defined(_WIN32)
#include <pic.h>
#endif
#include "typedefs.h"

typedef enum _BOOTPROTOCOL {
	READ = 0x39,
	READ_ACK,
	WRITE,
	WRITE_ACK,
	WRITE_OK,
	WRITE_FAILED,
	DATA_OK,
	DATA_FAILED,
	IDENTIFY,
	ID_ACK,
	DONE
}BOOTPROTOCOL;

#define BUFFER_SIZE		32
#define ROM_SIZE		8192

#if !defined(_WIN32)

#if defined(_16F870)
#define LOADER_START			0x0700
#elif defined(_16F873)||defined(_16F874)||defined(_16F873A)||defined(_16F874A)
#define LOADER_START			0x0F00
#elif defined(_16F876)||defined(_16F877)||defined(_16F876A)||defined(_16F877A)
#define LOADER_START			0x1F00
#else
#error This bootloader code is for 16F87x series microcontrollers
#endif
#define USER_START			LOADER_START
#define LOADER_SIZE			256									/* compiled size of loader (words) */
#define LOADER_LIMIT_LOW	(UINT16)(LOADER_START+4)			/* Low address of boot loader */
#define LOADER_ID			LOADER_LIMIT_LOW					/* id indicates program downloaded */
#define LOADER_LIMIT_HIGH	(UINT16)(LOADER_START+LOADER_SIZE)	/* High address of boot loader */
#define PROGRAMMED_ID		0xAA

// FOSC and BAUDRATE must be defined in compiler's command line
#define _SPBRG			(UBYTE)(FOSC/(16.0*BAUDRATE)-0.5)
#define DELAY_LOOP		(ULONG32)(FOSC)/2000UL

#define byte0(x)		(UBYTE)(*(((UBYTE*)&x)+0))
#define byte1(x)		(UBYTE)(*(((UBYTE*)&x)+1))
#define byte2(x)		(UBYTE)(*(((UBYTE*)&x)+2))
#define byte3(x)		(UBYTE)(*(((UBYTE*)&x)+3))

#endif	// _WIN32

#endif
